package order

// Type 表示訂單類型
type Type int64

const (
	UnDefine    Type = iota
	ProductType      // 商品訂單
)

// typeToStr 是 Type 對應的文字敘述
var typeToStr = map[Type]string{
	UnDefine:    "un_define",
	ProductType: "product",
}

// strToType 是文字轉回 Type 的對應
var strToType = map[string]Type{
	"un_define": UnDefine,
	"product":   ProductType,
}

// ToInt 將 Type 轉為 int
func (t Type) ToInt() int {
	return int(t)
}

// ToString 將 Type 轉為 string
func (t Type) ToString() string {
	if s, ok := typeToStr[t]; ok {
		return s
	}
	return "unknown"
}

// ParseType 從字串解析 Type，預設回傳 UnDefine
func ParseType(s string) Type {
	if t, ok := strToType[s]; ok {
		return t
	}
	return UnDefine
}
